from comk_django_plugin import PublicServer

from comk_django_account.views.dao.AccountDao import AccountDao


class AddAcount(PublicServer):
    '''
    添加账户

    '''

    def deal_request(self):
        data = self.request_data
        ad = AccountDao()
        ad.save_account(data=data)
        return self.build_success_response_data(response_data=True)



from alipay.aop.api.util.EncryptUtils import encrypt_content, decrypt_content
from django.db import models

from comk_django_account import get_account_aes_key


class Account(models.Model):
    '''
    app账户信息

    '''
    SaveTag = False

    appid = models.CharField(max_length=30, verbose_name='账户appid', primary_key=True)

    pri_key = models.TextField(verbose_name='私钥')
    own_public_key = models.TextField(verbose_name='个人公钥')
    other_public_key = models.TextField(verbose_name='对方公钥')

    description = models.CharField(max_length=50, verbose_name='账户描述', null=True, blank=True)
    create_time = models.DateTimeField(auto_now_add=True, verbose_name='添加时间')

    is_use = models.BooleanField(default=False, verbose_name='是否启用')
    account_level = models.CharField(max_length=3, default='1', verbose_name='账户等级')

    Field1 = models.CharField(max_length=50, verbose_name='保留字段1', null=True, blank=True)
    Field2 = models.CharField(max_length=50, verbose_name='保留字段2', null=True, blank=True)
    Field3 = models.CharField(max_length=50, verbose_name='保留字段3', null=True, blank=True)
    # encrypt_items记录哪些字段需要加密
    encrypt_items = ['pri_key', 'other_public_key', 'own_public_key']

    # 重写Model的save函数，实现当向数据库插入字段时自动加密的功能
    def save(self, *args, **kwargs):
        '''
        save方法重写。

        :param args:
        :param kwargs:
        :return:
        '''
        AES_KEY = get_account_aes_key()
        for item in self.encrypt_items:
            self.__setattr__(item, encrypt_content(
                super(Account, self).__getattribute__(item), encrypt_type='AES', encrypt_key=AES_KEY, charset='utf-8'))
        # 调用父类Model的save函数，进行真正的数据库插入操作
        super(Account, self).save(*args, **kwargs)

    def __getattribute__(self, item):
        '''
        注意，通过实例获取实例的属性（任何属性）时，会调用该方法。

        :param item:
        :return:
        '''
        AES_KEY = get_account_aes_key()
        try:
            if item in super(Account, self).__getattribute__(
                    'encrypt_items') and not super(Account, self).__getattribute__('SaveTag'):
                '''
                注意，这里只能通过 父类的 __getattribute__ 方法，获取属性值。
                如果用 self.__getattribute__ ，则会调用 自身的 __getattribute__ 方法，这样的话就会造成死循环。
                同时，也不能用 getattr(self, item) 方法，因为 getattr 的底层调用就是 __getattribute__ 方法。
                '''
                return decrypt_content(super(Account, self).__getattribute__(item), encrypt_type='AES',
                                       encrypt_key=AES_KEY, charset='utf-8')
            else:
                return super(Account, self).__getattribute__(item)
        except AttributeError as err:
            # 可能出现AttributeError异常，捕获后直接上抛即可
            raise err

    class Meta:
        verbose_name = "app信息（机密）"
        verbose_name_plural = verbose_name
        db_table = 'api_account'

    def __str__(self):
        return self.appid



# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2019-04-25 09:11
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Account',
            fields=[
                ('appid', models.CharField(max_length=30, primary_key=True, serialize=False, verbose_name='账户appid')),
                ('pri_key', models.TextField(verbose_name='私钥')),
                ('own_public_key', models.TextField(verbose_name='个人公钥')),
                ('other_public_key', models.TextField(verbose_name='对方公钥')),
                ('description', models.CharField(blank=True, max_length=50, null=True, verbose_name='账户描述')),
                ('create_time', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('is_use', models.BooleanField(default=False, verbose_name='是否启用')),
                ('account_level', models.CharField(default='1', max_length=3, verbose_name='账户等级')),
                ('Field1', models.CharField(blank=True, max_length=50, null=True, verbose_name='保留字段1')),
                ('Field2', models.CharField(blank=True, max_length=50, null=True, verbose_name='保留字段2')),
                ('Field3', models.CharField(blank=True, max_length=50, null=True, verbose_name='保留字段3')),
            ],
            options={
                'verbose_name': 'app信息（机密）',
                'verbose_name_plural': 'app信息（机密）',
                'db_table': 'api_account',
            },
        ),
        migrations.CreateModel(
            name='AccountUseInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ip', models.CharField(blank=True, max_length=50, null=True, verbose_name='ip地址')),
                ('method', models.CharField(blank=True, max_length=100, null=True, verbose_name='调用接口')),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='调用日期')),
                ('is_useful', models.BooleanField(default=False, verbose_name='有效调用')),
                ('Field1', models.CharField(blank=True, max_length=50, null=True, verbose_name='保留字段1')),
                ('Field2', models.CharField(blank=True, max_length=50, null=True, verbose_name='保留字段2')),
                ('Field3', models.CharField(blank=True, max_length=50, null=True, verbose_name='保留字段3')),
                ('app', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='comk_django_account.Account', verbose_name='账户')),
            ],
            options={
                'verbose_name': 'app调用信息',
                'verbose_name_plural': 'app调用信息',
                'db_table': 'api_useinfobyday',
            },
        ),
    ]



import json

from django.http import JsonResponse
from django.utils.decorators import method_decorator
from django.views import View
from django.views.decorators.csrf import csrf_exempt

from .Funs import FUNCTIONS


@method_decorator(csrf_exempt, name='dispatch')
class AcountView(View):
    '''
    添加账户接口

    '''

    def post(self, request):
        data = json.loads(request.body)
        method = data.get('method', '')
        if method in FUNCTIONS.keys():
            re = FUNCTIONS[method](request).deal_request()
            return JsonResponse(re)
        else:
            return JsonResponse({'msg': 'wrong request'})



import comk_django_account


def set_account_aes_key(account_aes_key):
    '''
    更改 ACCOUNT_AES_KEY

    :param account_aes_key:
    :return:
    '''
    if isinstance(account_aes_key, str) and len(account_aes_key) == 32:
        comk_django_account.ACCOUNT_AES_KEY = account_aes_key
    else:
        raise Exception('account_aes_key must be str and the length of account_aes_key must be 32')


def get_account_aes_key():
    '''
    获取 ACCOUNT_AES_KEY

    :return:
    '''
    return comk_django_account.ACCOUNT_AES_KEY



from django.contrib import admin
from .models import *


class AccountAdmin(admin.ModelAdmin):  # 注册CarouselInfoAdmin。注意，它继承的是object

    list_display = ['appid', 'description', 'create_time', 'is_use']
    list_exclude = ['pri_key', 'other_public_key', 'own_public_key']  # 列表隐藏
    search_fields = ['appid', 'description']
    list_filter = ['is_use', 'create_time']  # 添加过滤器功能
    exclude = ['pri_key', 'other_public_key', 'own_public_key']  # 设置隐藏字段。


admin.site.register(Account, AccountAdmin)


class AccountUseInfoAdmin(admin.ModelAdmin):  # 注册CarouselInfoAdmin。注意，它继承的是object

    list_display = ['app', 'ip', 'method', 'create_time']
    search_fields = ['ip', 'method']
    list_filter = ['app', 'create_time']  # 添加过滤器功能


admin.site.register(AccountUseInfo, AccountUseInfoAdmin)



from alipay.aop.api.util.SignatureUtils import get_sign_content, sign_with_rsa2, verify_with_rsa


def get_sign(pri_key, r_data):
    '''
    获取签名

    :param pri_key: 私钥
    :param r_data: 一个dict数据
    :return:
    '''
    sign_content = get_sign_content(r_data)
    return sign_with_rsa2(pri_key, sign_content, 'utf-8')


def check_sign(public_key, r_data):
    '''
    校验签名

    :param public_key: 公钥
    :param r_data: 一个dict数据
    :return:
    '''
    sign = r_data.get('sign')
    r_data.pop('sign')  # 在这里，data去掉了sign
    sign_content = get_sign_content(r_data).encode('utf-8')
    return verify_with_rsa(public_key, sign_content, sign)



# -*- coding: utf-8 -*-
# @Time : 2018/7/11 下午2:18
# @Author : ScrewMan
# @Site : 
# @File : permition_check.py
# @Software : PyCharm
import json
import logging
import re

from datetime import datetime

from alipay.aop.api.util.SignatureUtils import get_sign_content

from comk_django_account.views.dao.AccountDao import AccountDao
from comk_django_plugin import PublicServer, general_resolve_request

from comk_django_account.models import AccountUseInfo
from .sign_utils import get_sign, check_sign


def do_log_useinfo(request, account, method, is_useful):
    '''
    记录调用信息

    :param request:
    :param account:
    :param method:
    :param is_useful:
    :return:
    '''
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    AccountUseInfo.objects.create(app=account, ip=ip, method=method, is_useful=is_useful)


def check_interface_use_flag(interface_key, account_level):
    '''
    根据调用策略，检验能否再次调用（需要重写）

    :param interface_key:
    :param account_level:
    :return:
    '''
    # global_times = 10000
    # global_times = 3
    # from comk_django_account.views.utils.cache_utils import CacheUtils
    # cu = CacheUtils()
    # result = cu.GetInterfaceTimes(interface_key)
    # if result == None:
    #     return False
    # elif result > global_times:
    #     return False
    # else:
    #     return True
    return True


def do_Exception_log(request):
    '''
    记录错误日志

    :param request:
    :return:
    '''
    try:
        request_result = general_resolve_request(request)
        log = logging.getLogger('comk_exception_log')  # 加载记录器
        log.exception(request_result)
    except:
        pass


def request_to_response(function):
    """
    用户访问权限校验，以及业务正常失败处理

    :param func:
    :return:
    """

    def wrap(*args, **kwargs):
        request = args[0]  # 获取request
        ps = PublicServer(request)
        resp = {'code': '', 'msg': '', 'timestamp': datetime.now().strftime("%Y-%m-%d %H:%M:%S")}

        method = request.method
        if method != 'POST':
            resp['code'] = '2002'
            resp['msg'] = 'this server is only support POST(method),please try again'
            return ps.return_json_response(resp)

        body = request.body
        if not body:
            resp['code'] = '2005'
            resp['msg'] = 'data error,data is null'
            return ps.return_json_response(resp)
        try:
            data = json.loads(body)
        except json.decoder.JSONDecodeError:
            resp['code'] = '2006'
            resp['msg'] = 'data error,json-str is needed'
            return ps.return_json_response(resp)

        appid = data.get('appid')
        if not appid:
            resp['code'] = '3001'
            resp['msg'] = 'appid error,appid is null'
            return ps.return_json_response(resp)

        account = AccountDao().get_account_by_appid(appid)
        if not account:
            resp['code'] = '3002'
            resp['msg'] = 'account error,there is no account which appid : %s' % (appid)
            return ps.return_json_response(resp)

        if not account.is_use:
            resp['code'] = '3003'
            resp['msg'] = 'account error,the %s has no permission' % (appid)
            return ps.return_json_response(resp)

        sign = data.get('sign')

        if not sign:
            resp['code'] = '4001'
            resp['msg'] = 'sign error,sign is null'
            resp['sign'] = get_sign(account.pri_key, resp)
            return ps.return_json_response(resp)

        sign_result = False
        try:
            sign_result = check_sign(account.other_public_key, data)
        except Exception as e:
            pass

        if not sign_result:
            resp['code'] = '4002'
            resp['msg'] = 'sign error,please chcek the sign-str:%s' % (get_sign_content(data))
            resp['sign'] = get_sign(account.pri_key, resp)
            return ps.return_json_response(resp)

        method = data.get('method')
        timestamp = data.get('timestamp')
        content = data.get('content')
        if not (method and timestamp and content):
            resp['code'] = '4003'
            resp['msg'] = 'field error,method or timestamp or content is null'
            resp['sign'] = get_sign(account.pri_key, resp)
            return ps.return_json_response(resp)

        reg = re.compile(r'(^[0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2})$')
        reg_flag = reg.findall(timestamp)
        if not reg_flag:
            resp['code'] = '4004'
            resp['msg'] = 'timestamp error,%s is not a format timestamp' % (timestamp)
            resp['sign'] = get_sign(account.pri_key, resp)
            return ps.return_json_response(resp)

        if not isinstance(content, dict):
            resp['code'] = '4006'
            resp['msg'] = 'the content(%s) is not dict' % (content)
            resp['sign'] = get_sign(account.pri_key, resp)
            return ps.return_json_response(resp)

        account_level = account.account_level
        check_pass = check_interface_use_flag(appid + '_' + method, account_level)
        do_log_useinfo(request, account, method, check_pass)
        if not check_pass:
            resp['code'] = '4007'
            resp['msg'] = 'the appid(%s) method(%s) too much times today' % (appid, method)
            resp['sign'] = get_sign(account.pri_key, resp)
            return ps.return_json_response(resp)

        try:
            request.session['account_level'] = account_level
            result = function(*args, **kwargs)
            result['sign'] = get_sign(account.pri_key, result)
            return ps.return_json_response(result)

        except Exception as e:
            resp['code'] = '9999'
            resp['msg'] = 'system error'
            resp['sign'] = get_sign(account.pri_key, resp)
            do_Exception_log(request)
            return ps.return_json_response(resp)

    return wrap



from comk_django_plugin import PublicDao
from django.db import models

from comk_django_account.models import Account


class AccountDao(PublicDao):
    def __init__(self, model: models.Model = Account):
        super().__init__(model)

    def get_account_by_appid(self, appid):
        key_filter = dict(
            appid=appid
        )
        return self.get_objs(key_filter=key_filter, get_flag=True)

    def save_account(self, data):
        account = self.return_obj(data, pk=data['appid'])
        account.SaveTag = True
        account.is_use = True
        account.save()
        return account



from datetime import datetime, timedelta

from django.core.cache import cache


class CacheUtils():
    def __init__(self):
        pass

    def GetInterfaceTimes(self, key):
        '''
        获取接口调用次数

        :param hpzl:
        :param hphm:
        :return:
        '''
        value = cache.get(key)

        if value:
            cache.incr(key)
            return cache.get(key)
        else:
            now = datetime.now()
            end_time = datetime(now.year, now.month, now.day, 0, 0, 0) + timedelta(days=1)
            countdown_time = int((end_time - now).total_seconds())
            cache.set(key, 1, countdown_time)
            return cache.get(key)



#!/usr/bin/env python
# coding=utf-8

from setuptools import setup, find_packages

setup(
    name='comk_django_account',  # 包名
    version='1.0.1',  # 版本
    description=(  # 简述
        'comk个人开发的account模型，用于构建RSA-SHA256验证的权限系统'
    ),
    long_description=open('README.rst').read(),  # 长描述， 必须是rst（reStructuredText )格式的，
    author='comk',  # 作者
    author_email='1943336161@qq.com',  # 作者邮箱
    maintainer='comk',  # 维护人
    maintainer_email='1943336161@qq.com',  # 维护人邮箱
    license='BSD License',  # 协议
    packages=find_packages(),  # 申明你的包里面要包含的目录，可以让setuptools自动决定要包含哪些包
    platforms=["all"],
    url=None,  # 项目的网址，一般都是github的url
    classifiers=[  # 运行环境
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
        # 'Programming Language :: Python :: 2',
        # 'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries'
    ],
    install_requires=[  # 依赖包，安装包时pip会自动安装
        'django',
    ]
)



from django.db import models

from comk_django_account.models import Account


class AccountUseInfo(models.Model):
    '''
    用户每日调用信息

    '''
    app = models.ForeignKey(Account, verbose_name='账户', null=True, blank=True)
    ip = models.CharField(max_length=50, verbose_name='ip地址', null=True, blank=True)
    method = models.CharField(max_length=100, verbose_name='调用接口', null=True, blank=True)
    create_time = models.DateTimeField(auto_now_add=True, verbose_name='调用日期', null=True, blank=True)
    is_useful = models.BooleanField(default=False, verbose_name='有效调用')

    Field1 = models.CharField(max_length=50, verbose_name='保留字段1', null=True, blank=True)
    Field2 = models.CharField(max_length=50, verbose_name='保留字段2', null=True, blank=True)
    Field3 = models.CharField(max_length=50, verbose_name='保留字段3', null=True, blank=True)

    class Meta:
        verbose_name = "app调用信息"
        verbose_name_plural = verbose_name
        db_table = 'api_useinfobyday'

    def __str__(self):
        return self.app



import json

import os
import django

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "base_service.settings")  # 配置django的setting文件
django.setup()  # 加载django程序

from django.test import Client

c = Client()  # 构造Client，一个全局变量


# c.login(username='2088102169075850', password='2088102169075850')


class APITests():  # 这里应为需要用到原数据库的数据，因此，不继承TestCase

    def test_gataway(self, data):
        '''
        测试 /acount_action/
        预期：{'code': '1000', 'data_type': '1', 'response_data': True, 'msg': ''}

        :return:
        '''
        json_data = json.dumps(data)  # 因为项目的API接收的是json字符串，因此需要进行一次转化
        response = c.post('/base_service/acount_action/', json_data,
                          content_type='application/json')  # 同时注意设置content_type类型
        return response


def main_t():
    '''
    使用 django 的测试模块进行测试

    :return:
    '''
    api_t = APITests()
    pri_key = 'MIIEpgIBAAKCAQEAxrJ5XqZM5NgHoM8/05yOeWNLuoyzYLW4qgLPdCOUYJ8h2tOOoVV6BE+dpUnm2mimuBIwOnCe1s7ejN+NIxtPchECNE084yqyovFVEqODYOquLXwOkLK+Wd4M9tFs331rZznvXiXOC7nP1MOs5c3qK6wFc6YYfEFJTR+m96FiDPepU6bAovBa/o4buhTdqGTQU/8BlWUY6CDn/o7gwPqeGpYhsbcnMzdwPaqLTKfzTovqPGvP32ejzfZe4OKRFXYxPRDJcP9aMl41yL/iMcUmPuSZ8c5f8KVMmkmUun4VtgjtfRqZgini+bOk9Fuiqju4qjAcDSxD8MpV5nZM57EtawIDAQABAoIBAQCSEhSdm13+Aj6hXprKg1ZOMMw0SUl5eD5YZZaDB8EFwxbNWyeIvWDlGG6JW6nluHeP7HceDzsGKbB8GcAVJfeioJRhRMLVMcd/eDEVcbWcowoHECNZBr5fLJmVPWJvbjFuAq5RJTOzdRcvj+ZkTpuvHVgoq9tmRUyQ8Qr7Et4VLxQYuBtosYjWyOnn5rNxtM+Wkps4bfgxpA59qGf9SSsTRXJBMFXuUe37304eETg2Knhd0szKH92AUMVFGXufiYqsGMes3XQ8ZY9cIVIqz48wvZV4Wp6VGOwCt8Tq9ZIbYzlet68b8GNb1eqms08GJ0Utmeae8KDRN42VDDsl4E5JAoGBAPHDWpgVbJqYBDTyPy+TE8JiwnBLIhy/5CadHROsCVCWVhSF7z8Sjqe18b7nXbV2lAF5OprPCOdLZ7R/WfmMGmiZV7b6ki9pSPp5jjWQchoo0XZHibVz5lOyU2Ez8561BLCwqDjoHPGDFmaM7HKZ2bLGNldczY48QFGt46irm991AoGBANJl46QM9wqc3YsQoG0UsY6YtP/hUZPx+W6y0Pkw4rMHbtF4Y1eruFsKdZ6iN+R78KB7KqWrHhQBuWo+AN1qHpY7a+ZYqivNGIGPn0T2dYQYg85YpZReXa0GkZo0ln/ngokqMWIg9UktflJiiRT9OeIHwEvOp9tiW7iK7Dc+RR1fAoGBAOp7KYO1y8bIx9O+XHA8+v4eoS0egfBWYObenVP2GHazyLtRB7Epw3h/6/7XEbktT/F57dA2GxGRvfx/Q/nX28k1aLCMqHoZaHGescHb3f7nNfi8+6Akz/C/VUVCiPeV4/2m6RGAdon7NR+zcKTZ+R3+RrhGxws7/rn5qINwJdb9AoGBAMtVzxfXs9Tc3zsr2W8yRZsjOCHnNzj87OwwAsghl6Yf89ibOZ8cBTJvPFcQPWYi5d2iyweLBameNPxZaeqSSXc1fEUEwYlNUHa+P9WdPttn+dItV/C49l1m8MaQNjbsnfpD/a/xXDzZq7ChLAxN4mfwx8Y7tXvPZgdGs518H72PAoGBANTE1lp10Q4b8AFgyHv+7FU3tnGmXBAnMglyuTt2UJQSYNJEWOX5UZ6IbgJdnnYS3VnOtch7URgibfY1LapWXQFzmDMiud0scQbEaaiT0+ysRFetc/rpADf46O4j/c0GXAYh5WsPrtV+MxfIg2P9uJQhWsL0TyZfvh0w6hx7bPwU'
    own_public_key = 'MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAxrJ5XqZM5NgHoM8/05yOeWNLuoyzYLW4qgLPdCOUYJ8h2tOOoVV6BE+dpUnm2mimuBIwOnCe1s7ejN+NIxtPchECNE084yqyovFVEqODYOquLXwOkLK+Wd4M9tFs331rZznvXiXOC7nP1MOs5c3qK6wFc6YYfEFJTR+m96FiDPepU6bAovBa/o4buhTdqGTQU/8BlWUY6CDn/o7gwPqeGpYhsbcnMzdwPaqLTKfzTovqPGvP32ejzfZe4OKRFXYxPRDJcP9aMl41yL/iMcUmPuSZ8c5f8KVMmkmUun4VtgjtfRqZgini+bOk9Fuiqju4qjAcDSxD8MpV5nZM57EtawIDAQAB'
    other_public_key = 'MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAw2trVpIaMbXjD72Hm8ZylSqJ+UG6nbEf7lhhKvurq+l0XhrFlls6h9PfbW2R5i5XOV4ptnn/zhpD8uwUJjgIYYj0s+UNqNj9Aa8dGXM8Ebav1ph1r7eZ7ibWpM9mzT3l1LucqaBzciDPuv4cA+inulR/CAByZYbkzb+aHQvDOZk77gZyvLMcqAlfg8pjAmDdkseDIBGawbbY7pOQ/ZawmwcGAFjBEAzq1e8em/PM3coKil3sogoIEoNCQBliL5iTMXXcUQn8NzGVVxVjPUyRawmsGiNJev+RyqEA3NWiRNV8a1jqVdz2cs/kLUtxiBOzDvQi1qu+ARMF6u2fUg6hRwIDAQAB'
    request_data = dict(
        method='account.add',
        description='测',
        appid='381',
        pri_key=pri_key,
        own_public_key=own_public_key,
        other_public_key=other_public_key,
    )

    result = api_t.test_gataway(request_data).json()  # 运行测试程序
    print('--------------------')
    print(result)
    print('--------------------')
    pass


if __name__ == '__main__':
    main_t()
